@@ -163,6 +163,18 @@ bootstrap_remote_fedora()
         && break
         retry_or_fail
     done
+
+    packages="
+        expat
+        "
+
+    retry_init
+    while true; do
+        # shellcheck disable=SC2086
+        dnf update -q -y ${packages} \
+        && break
+        retry_or_fail
+    done
 }
 
 bootstrap_remote_freebsd()
@@ -187,12 +199,6 @@ bootstrap_remote_freebsd()
         # Declare platform/python version combinations which do not have supporting OS packages available.
         # For these combinations ansible-test will use pip to install the requirements instead.
         case "${platform_version}/${python_version}" in
-            13.5/3.11)
-                # defaults available
-                ;;
-            14.2/3.11)
-                # defaults available
-                ;;
             *)
                 # just assume nothing is available
                 jinja2_pkg=""  # not available
